function [U,S,V] = randSVD(A,k,d)
% Description: Implementation of randomized SVD algorithm as described in:
% N. Halko, P.-G. Martinsson, J.A. Tropp, 
% Finding structure with randomness: stochastic algorithms 
% for constructing approximate matrix decompositions, 
% SIAM Rev. 53(2), 217-288 (2011)
%
% Usage: [U,S,V] = randSVD(A,k,d)
% Input: A - matrix needed to be approximate
%        k - rank k matrix approximation of A
%        d - an over-sampling parameter
% Output:
%    U,S,V - approximate SVD variables 

[m,n] = size(A);
l = k+d;

% Generate an oversampling matrix 
Omega = randn(n,l)/sqrt(l); 

% Get a new matrix of much smaller size, abeit similar subspace as A
Y = A*Omega;

% Compute QR factorization
[Q,R] = qr(Y,0);

% Generate a smaller matrix and calculate SVD
B = Q'*A;
[Uhat,S,V] = svd(B,0);

% Rank k approximation
U = Q*Uhat;
U = U(:,1:k); S = S(1:k,1:k); V = V(:,1:k);

